'use strict';

exports.__esModule = true;
exports.isAnimating = exports.strip = exports.isNumber = undefined;

var _objectWithoutProperties2 = require('babel-runtime/helpers/objectWithoutProperties');

var _objectWithoutProperties3 = _interopRequireDefault(_objectWithoutProperties2);

var _map = require('babel-runtime/core-js/map');

var _map2 = _interopRequireDefault(_map);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _isNan = require('babel-runtime/core-js/number/is-nan');

var _isNan2 = _interopRequireDefault(_isNan);

var _weakMap = require('babel-runtime/core-js/weak-map');

var _weakMap2 = _interopRequireDefault(_weakMap);

var _raf = require('raf');

var _raf2 = _interopRequireDefault(_raf);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _easingJs = require('easing-js');

var _easingJs2 = _interopRequireDefault(_easingJs);

var _performanceNow = require('performance-now');

var _performanceNow2 = _interopRequireDefault(_performanceNow);

var _lodash = require('lodash.throttle');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var privates = new _weakMap2.default();

var isNumber = exports.isNumber = function isNumber(obj) {
    return typeof obj === 'number' && !(0, _isNan2.default)(obj);
};

var strip = exports.strip = function strip(number) {
    return parseFloat(number.toPrecision(12));
};

var isAnimating = exports.isAnimating = function isAnimating(animations) {
    for (var _iterator = animations, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : (0, _getIterator3.default)(_iterator);;) {
        var _ref;

        if (_isArray) {
            if (_i >= _iterator.length) break;
            _ref = _iterator[_i++];
        } else {
            _i = _iterator.next();
            if (_i.done) break;
            _ref = _i.value;
        }

        var _ref2 = _ref;
        var animation = _ref2[1];

        if (animation.isAnimating) return true;
    }
    return false;
};

var scheduleAnimation = function scheduleAnimation(context) {
    (0, _raf2.default)(function () {
        var animations = privates.get(context);
        var currentTime = (0, _performanceNow2.default)();
        var shouldUpdate = false;
        animations && animations.forEach(function (animation, name) {
            var isFunction = typeof name === 'function';
            if (!animation.isAnimating) return;
            var duration = animation.duration;
            var easing = animation.easing;
            var endValue = animation.endValue;
            var startTime = animation.startTime;
            var startValue = animation.startValue;

            var deltaTime = currentTime - startTime;
            if (deltaTime >= duration) {
                (0, _extends3.default)(animation, { isAnimating: false, startTime: currentTime, value: endValue });
            } else {
                animation.value = strip(_easingJs2.default[easing](deltaTime, startValue, endValue - startValue, duration));
            }
            shouldUpdate = shouldUpdate || !isFunction;
            if (isFunction) name(animation.value);
        });
        if (animations && isAnimating(animations)) scheduleAnimation(context);
        if (shouldUpdate) context.forceUpdate();
    });
};

var Back2Top = function (_React$Component) {
    (0, _inherits3.default)(Back2Top, _React$Component);

    function Back2Top(props) {
        (0, _classCallCheck3.default)(this, Back2Top);

        var _this = (0, _possibleConstructorReturn3.default)(this, _React$Component.call(this, props));

        _this.animate = _this.animate.bind(_this);
        _this.scrollToTop = _this.scrollToTop.bind(_this);
        _this.updateScroll = _this.updateScroll.bind(_this);
        _this.shouldAnimate = _this.shouldAnimate.bind(_this);
        _this.animate = _this.animate.bind(_this);
        _this.scrollToTop = _this.scrollToTop.bind(_this);
        _this.isFirefox = _this.isFirefox.bind(_this);
        _this.getScrollTop = _this.getScrollTop.bind(_this);
        _this.setScrollTop = _this.setScrollTop.bind(_this);
        _this.state = {
            visible: false
        };
        return _this;
    }

    Back2Top.prototype.componentDidMount = function componentDidMount() {
        this.throttledUpdateScroll = (0, _lodash2.default)(this.updateScroll, 100);
        window.addEventListener('scroll', this.throttledUpdateScroll);
    };

    Back2Top.prototype.componentWillUnmount = function componentWillUnmount() {
        privates.delete(this);
        window.removeEventListener('scroll', this.throttledUpdateScroll);
    };

    Back2Top.prototype.shouldAnimate = function shouldAnimate() {
        return true;
    };

    Back2Top.prototype.animate = function animate(name, endValue, duration) {
        var options = arguments.length <= 3 || arguments[3] === undefined ? {} : arguments[3];

        var animations = privates.get(this);
        if (!animations) {
            privates.set(this, animations = new _map2.default());
        }
        var animation = animations.get(name);
        var shouldAnimate = this.shouldAnimate() && options.animation !== false;
        if (!animation || !shouldAnimate || !isNumber(endValue)) {
            var easing = options.easing || 'linear';
            var startValue = isNumber(options.startValue) && shouldAnimate ? options.startValue : endValue;
            animation = { duration: duration, easing: easing, endValue: endValue, isAnimating: false, startValue: startValue, value: startValue };
            animations.set(name, animation);
        }
        if (!duration) {
            (0, _extends3.default)(animation, { endValue: endValue, value: endValue });
            animations.set(name, animation);
        }
        if (animation.value !== endValue && !animation.isAnimating) {
            if (!isAnimating(animations)) scheduleAnimation(this);
            var startTime = 'startTime' in options ? options.startTime : (0, _performanceNow2.default)();
            duration = duration || animation.duration;
            var _easing = options.easing || animation.easing;
            var _startValue = animation.value;
            (0, _extends3.default)(animation, { isAnimating: true, endValue: endValue, startValue: _startValue, startTime: startTime, duration: duration, easing: _easing });
        }
        return animation.value;
    };

    Back2Top.prototype.updateScroll = function updateScroll() {
        var visibilityHeight = this.props.visibilityHeight;

        this.setState({ visible: this.getScrollTop() > (visibilityHeight || Back2Top.VISIBILITY_HEIGHT) });
    };

    Back2Top.prototype.scrollToTop = function scrollToTop(e) {
        var _this2 = this;

        if (e) e.preventDefault();
        var scrollDuration = this.props.scrollDuration;

        this.animate(function (value) {
            return _this2.setScrollTop(value);
        }, 0, scrollDuration || Back2Top.SCROLL_DURATION, { startValue: this.getScrollTop() });
        if (this.props.onClick) {
            this.props.onClick(e);
        }
    };

    Back2Top.prototype.isFirefox = function isFirefox() {
        return navigator.userAgent.toLowerCase().indexOf('firefox') !== -1;
    };

    Back2Top.prototype.getScrollTop = function getScrollTop() {
        return this.isFirefox() ? document.documentElement.scrollTop : document.body.scrollTop;
    };

    Back2Top.prototype.setScrollTop = function setScrollTop(value) {
        document.body.scrollTop = value;
        document.documentElement.scrollTop = value;
    };

    Back2Top.prototype.render = function render() {
        var visible = this.props.alwaysVisible || this.state.visible;
        var _props = this.props;
        var href = _props.href;
        var style = _props.style;
        var target = _props.target;
        var onClick = _props.onClick;
        var className = _props.className;
        var fadeDuration = _props.fadeDuration;
        var options = (0, _objectWithoutProperties3.default)(_props, ['href', 'style', 'target', 'onClick', 'className', 'fadeDuration']);

        var opacity = this.animate('opacity', visible ? 1 : 0, fadeDuration || Back2Top.FADE_DURATION);
        if (opacity === 0) return false;
        return _react2.default.createElement(
            'a',
            (0, _extends3.default)({
                className: className || 'back-to-top'
            }, options, {
                'aria-label': 'Back to top',
                style: { display: 'inline', opacity: opacity },
                onClick: this.scrollToTop }),
            this.props.children
        );
    };

    return Back2Top;
}(_react2.default.Component);

Back2Top.propTypes = {
    // Make the button visible
    alwaysVisible: _react2.default.PropTypes.bool,
    // Duration of fade effect
    fadeDuration: _react2.default.PropTypes.number,
    // Duration of scroll-to-top effect
    scrollDuration: _react2.default.PropTypes.number,
    // Height of button to become visible
    visibilityHeight: _react2.default.PropTypes.number
};
Back2Top.FADE_DURATION = 300;
Back2Top.SCROLL_DURATION = 800;
Back2Top.VISIBILITY_HEIGHT = 400;


Back2Top.prototype.displayName = 'Back2Top';

exports.default = Back2Top;